#Power law analysis script
#See the following paper for guidelines on how to use powerlaw package: Alstott, J., Bullmore, E., & Plenz, D. (2014). powerlaw: a Python package for analysis of heavy-tailed distributions. PloS one, 9(1), e85777. 

import powerlaw
import numpy 
import matplotlib
import csv
import os
import pandas as pd

data = pd.read_csv(""); # insert location of file here

#specify that this is discrete data, not continuous
fit = powerlaw.Fit(data.Degree, discrete = True)

fit.power_law.alpha
fit.power_law.sigma
fit.xmin
data.Degree.max()

fit = powerlaw.Fit(data.Degree, discrete = True, xmin =0) #set xmin, based on above xmin output

# Below is an experpt from Alstott, Bullmore and Plenz (2014), explaining the below output:
# "R is the loglikelihood ratio between the two candidate distributions. This number will be positive if the data is 
# more likely in the first distribution, and negative if the data is more likely in the second distribution. 
# The significance value for that direction is p. The normalized_ratio option normalizes R by its standard deviation, 
# R/( stdev* srt n) The normalized ratio is what is directly used to calculate p."
fit.distribution_compare('power_law', 'lognormal_positive',normalized_ratio=True)
fit.distribution_compare('power_law', 'exponential',normalized_ratio=True)

# plot all values of data x>1
figPDF = powerlaw.plot_pdf(data.Degree, color='b', label='Empirical data')
fit.lognormal_positive.plot_pdf(ax=figPDF, color='g', linestyle='--', label='Lognormal fit')
fit.power_law.plot_pdf(ax=figPDF, color='r', linestyle='--', label='Power law fit')
fit.exponential.plot_pdf(ax=figPDF, color='k', linestyle='--', label='Exponential fit')
#display fit line if the xmin starts at 1
plt.xlim([0, 1000])
handles, labels = figPDF.get_legend_handles_labels()
figPDF.legend(handles, labels, loc=3)
figPDF.set_ylabel("p(X)")
figPDF.set_xlabel(r"Degree")
figname = 'FigPDF_R10M8'
savefig(figname+'.pdf', bbox_inches='tight')

#plot data starting at xmin
fit = powerlaw.Fit(data.Degree, discrete=True)
fit.distribution_compare('power_law', 'lognormal')
fig = fit.plot_pdf(linewidth=3, label='Empirical Data')
fit.lognormal_positive.plot_pdf(ax=fig, color='g', linestyle='--', label='Lognormal fit')
fit.power_law.plot_pdf(ax=fig, color='r', linestyle='--', label='Power law fit')
fit.exponential.plot_pdf(ax=fig, color='k', linestyle='--', label='Exponential fit')
plt.xlim([0, data.Degree.max()*2])
fig.set_ylabel(u"p(X)")
fig.set_xlabel("Degree")
handles, labels = fig.get_legend_handles_labels()
fig.legend(handles, labels, loc=3)
figname = 'FigLognormal_R10M8'
savefig(figname+'.pdf', bbox_inches='tight')

#calculate Gini coefficient
#credit for Gini calculation code to Zhiya Zuo: https://zhiyzuo.github.io/Plot-Lorenz/

def gini(arr):
    ## first sort
    sorted_arr = arr.copy()
    sorted_arr.sort()
    n = arr.size
    coef_ = 2. / n
    const_ = (n + 1.) / n
    weighted_sum = sum([(i+1)*yi for i, yi in enumerate(sorted_arr)])
    return coef_*weighted_sum/(sorted_arr.sum()) - const_

gini(data.Degree)

## Plot Histograms - y logged and unlogged
import matplotlib.pyplot as plt
import numpy as np
from matplotlib import colors

data = pd.read_csv(""); # point to csv file 
n, bins, patches = plt.hist(data.degree, 50, normed=0, facecolor='green', alpha=0.75)
plt.xlabel('Degree')
plt.ylabel('Count')
plt.yscale('log')
plt.xlim([0, 700])
plt.title(r'LogY Histogram of RealWorld')
plt.grid(True)
figname = 'Histogram_LogY_RealWorld'
savefig(figname+'.pdf', bbox_inches='tight')
plt.show()

n, bins, patches = plt.hist(data.degree, 50, normed=0, facecolor='green', alpha=0.75)
plt.xlabel('Degree')
plt.ylabel('Count')
plt.title(r'Histogram of RealWorld')
plt.xlim([0, data.degree.max()*1.1])
plt.grid(True)
figname = 'Histogram_RealWorld'
savefig(figname+'.pdf', bbox_inches='tight')
plt.show()

## Lock the X and Y axes for comparison among plots
data = pd.read_csv(""); #point to csv file
n, bins, patches = plt.hist(data.degree, 50, normed=0, facecolor='green', alpha=0.75)
plt.xlabel('Degree')
plt.ylabel('Log Count')
plt.yscale('log')
plt.xlim([0, 700])
#plt.ylim([0, 10000])
plt.grid(True)
plt.title(r'R10M10')
figname = 'Locked_Histogram_LogY_R10M10'
savefig(figname+'.png', bbox_inches='tight')

plt.show()






