# this script will iterate through a set of graphml objects in a set directory and calculate 
# the following network metrics: NumberNodes, AvgClusteringCoeff, Modularity, eccentricity_average, ecc_stdev, Diameter, degree_of_fragmentation, avg_degree, max_degree, Avg_short_path_length 

import community
import networkx as nx
import random
from random import randint
import time
import statistics
import os

global filenum 
filenum = 1
global graphs

directory = '' #set directory here
for filename in os.listdir(directory):
    
    if filename.endswith(".graphml"):
        print(filenum)
        g = nx.read_gexf(directory + "\\OnlineWorld_R10M10_" + str(filenum) +".graphml")
        #print(directory + "OnlineWorld_R10M10_" + str(filenum) +".graphml")  
        print("Number nodes: ",g.number_of_nodes())
        #print(nx.degree_centrality(g))
        #list isolates
        print("Isolates: ",list(nx.isolates(g)))
        #Remove isolates
        g.remove_nodes_from(list(nx.isolates(g)))
        print("Connected Components: ",nx.number_connected_components(g))

        Gcc = nx.connected_component_subgraphs(g) 
        # this is a list of graphs, Gcc[0] is the most important here 
        [ len(i) for i in Gcc ] 
        graphs = list(nx.connected_component_subgraphs(g))
        #print number of nodes in other subgraph
        #print(graphs[1])

		#only calculate metrics on the primary component, i.e., remove isolates and other minute components
        if nx.number_connected_components(g) > 1:
            while nx.number_connected_components(g) > 1:
                graphs = list(nx.connected_component_subgraphs(g))
                g.remove_nodes_from(graphs[1])
 
        Diameter = nx.diameter(g)
        print("Diameter: ", Diameter)
        FinalNodes= g.number_of_nodes()
        
        eccdic = nx.eccentricity(g)
        ecc_sum = sum(eccdic.values())
        ecc_length = len(eccdic)
        eccentricity_average = ecc_sum/ecc_length
        print(eccentricity_average)
        #print ("Avg Eccentricity: ", nx.eccentricity(g))
        
        ecc_stdev = statistics.stdev(eccdic.values())
        print(ecc_stdev)
 
        avg_degree = statistics.mean(nx.degree(g).values())
        print(avg_degree)
        
		max_degree = max(nx.degree(g).values())
        print(max_degree)

        part = community.best_partition(g)
        modularity = community.modularity(part,g)
        print(modularity)

        AverageClustering = nx.average_clustering(g) 
        print(AverageClustering)
        #print(nx.degree_centrality(g))   
		
		##Credit for the below degree of fragmentation calculation: https://stats.stackexchange.com/questions/121645/borgatti-key-player-problem-kpp-in-python, employing Borgatti's
        ## http://www.bebr.ufl.edu/sites/default/files/Borgatti%20-%202006%20-%20Identifying%20sets%20of%20key%20players%20in%20a%20social%20networ.pdf

        n = g.number_of_nodes()
        denominator = n * (n - 1)
        sum_of_distance = 0.0
        distance_matrix = nx.all_pairs_shortest_path_length(g)
        for source, destinations in distance_matrix.items():
            for destination, length in destinations.items():
                if not source == destination:
                    sum_of_distance += 1 / length
        degree_of_fragmentation = (2 * sum_of_distance) / denominator
        print(degree_of_fragmentation)        
        
        Avg_shortest_path_length= nx.average_shortest_path_length(g)  
        
        write_time = time.strftime("%Y%m%d-%H%M%S")
        # open file for writing
        fileObj=open("Network_Report_" + write_time + ".csv", 'w')  # 'w' open for writing mode
        lineList = []
        lineList.append("NumberNodes, AvgClusteringCoeff, Modularity, eccentricity_average, ecc_stdev, Diameter, degree_of_fragmentation, avg_degree, max_degree, Avg_short_path_length \n")
        oneLine = " %0.6f, %0.6f, %0.6f, %0.6f, %0.6f, %0.6f, %0.6f, %0.6f, %0.6f, %0.6f\n" %(FinalNodes, AverageClustering, modularity, eccentricity_average, ecc_stdev, Diameter, degree_of_fragmentation, avg_degree, max_degree, Avg_shortest_path_length)
        lineList.append(oneLine)
        fileObj.writelines(lineList)
        fileObj.close()
        
        filenum = filenum + 1
        continue


